@@ -43,7 +43,7 @@ package Mail::SpamAssassin::Message::Metadata::Received; 1;
 package Mail::SpamAssassin::Message::Metadata;
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
 use Mail::SpamAssassin::Dns;
@@ -53,15 +53,16 @@ use Mail::SpamAssassin::Constants qw(:ip);
 # ---------------------------------------------------------------------------
 
 sub parse_received_headers {
-  my ($self, $permsgstatus, $msg) = @_;
-
-  my $suppl_attrib = $msg->{suppl_attrib};  # out-of-band info from a caller
+  my ($self, $pms, $msg) = @_;
 
   # a caller may assert that a message is coming from inside or from an
   # authenticated roaming users; this info may not be available in mail
   # header section, e.g. in case of nonstandard authentication mechanisms
   my $originating;  # boolean
-  $originating = $suppl_attrib->{originating}  if ref $suppl_attrib;
+  if (exists $msg->{suppl_attrib}->{originating}) {
+    $originating = $msg->{suppl_attrib}->{originating} || 0;
+    dbg("metadata: set originating from suppl_attrib: %s", $originating);
+  }
 
   $self->{relays_trusted} = [ ];
   $self->{num_relays_trusted} = 0;
@@ -84,12 +85,14 @@ sub parse_received_headers {
   $self->{last_trusted_relay_index} = -1;	# last counting from the top,
   $self->{last_internal_relay_index} = -1;	# first in time
 
+  $self->{allow_mailfetch_markers} = 1;         # This needs to be set for the
+                                                # first Received: header
   # now figure out what relays are trusted...
-  my $trusted = $permsgstatus->{main}->{conf}->{trusted_networks};
-  my $internal = $permsgstatus->{main}->{conf}->{internal_networks};
-  my $msa = $permsgstatus->{main}->{conf}->{msa_networks};
-  my $did_user_specify_trust = $permsgstatus->{main}->{conf}->{trusted_networks_configured};
-  my $did_user_specify_internal = $permsgstatus->{main}->{conf}->{internal_networks_configured};
+  my $trusted = $pms->{main}->{conf}->{trusted_networks};
+  my $internal = $pms->{main}->{conf}->{internal_networks};
+  my $msa = $pms->{main}->{conf}->{msa_networks};
+  my $did_user_specify_trust = $pms->{main}->{conf}->{trusted_networks_configured};
+  my $did_user_specify_internal = $pms->{main}->{conf}->{internal_networks_configured};
   my $in_trusted = 1;
   my $in_internal = 1;
   my $found_msa = 0;
@@ -123,8 +126,7 @@ sub parse_received_headers {
   # Now add the single line headers like X-Originating-IP. (bug 5680)
   # we convert them into synthetic "Received" headers so we can share
   # code below.
-  for my $header (@{$permsgstatus->{main}->{conf}->{originating_ip_headers}})
-  {
+  foreach my $header (@{$pms->{main}->{conf}->{originating_ip_headers}}) {
     my $str = $msg->get_header($header);
     next unless ($str && $str =~ m/($IP_ADDRESS)/);
     push @hdrs, "from X-Originating-IP: $1\n";
@@ -238,11 +240,11 @@ sub parse_received_headers {
 
     if ($in_trusted) {
       push (@{$self->{relays_trusted}}, $relay);
-      $self->{allow_fetchmail_markers} = 1;
+      $self->{allow_mailfetch_markers} = 1;
       $self->{last_trusted_relay_index}++;
     } else {
       push (@{$self->{relays_untrusted}}, $relay);
-      $self->{allow_fetchmail_markers} = 0;
+      $self->{allow_mailfetch_markers} = 0;
     }
 
     if ($in_internal) {
@@ -332,7 +334,7 @@ sub parse_received_line {
   my $by = '';
   my $id = '';
   my $ident = '';
-  my $envfrom = '';
+  my $envfrom = undef;
   my $mta_looked_up_dns = 0;
   my $IP_ADDRESS = IP_ADDRESS;
   my $IP_PRIVATE = IP_PRIVATE;
@@ -404,8 +406,10 @@ sub parse_received_line {
   # with ASMTP (Authenticated SMTP) is used by Earthlink, Exim 4.34, and others
   # with HTTP should only be authenticated webmail sessions
   # with HTTPU is used by Communigate Pro with Pronto! webmail interface
-  # IANA registry: http://www.iana.org/assignments/mail-parameters/mail-parameters.xhtml
-  if (/ by / && / with ((?:ES|L|UTF8S|UTF8L)MTPS?A|ASMTP|HTTPU?)(?: |;|$)/i) {
+  # with HTTPS is used by Horde adjusts the Received header to say "HTTPS" when
+  # a connection is made over HTTPS
+  # IANA registry: https://www.iana.org/assignments/mail-parameters/mail-parameters.xhtml
+  if (/ by / && / with ((?:ES|L|UTF8S|UTF8L)MTPS?A|ASMTP|HTTP[SU]?)(?: |;|$)/i) {
     $auth = $1;
   }
   # GMail should use ESMTPSA to indicate that it is in fact authenticated,
@@ -430,8 +434,9 @@ sub parse_received_line {
     $auth = 'CriticalPath';
   }
   # Postfix 2.3 and later with "smtpd_sasl_authenticated_header yes"
-  elsif (/\) \(Authenticated sender: \S+\) by \S+ \(Postfix\) with /) {
-    $auth = 'Postfix';
+  # Normally $1 is "Postfix", but could be changed with mail_name (Bug 5646)
+  elsif (/\) \(Authenticated sender: \S+\) by \S+ \(([^\)]+)\) with /) {
+    $auth = $1 eq 'Postfix' ? $1 : "Postfix ($1)";
   }
   # Communigate Pro - Bug 6495 adds HTTP as possible transmission method
   # 	Bug 7277: XIMSS used by Pronto and other custom apps, IMAP supports XMIT extension
@@ -498,6 +503,21 @@ sub parse_received_line {
       }
     }
 
+    # Microsoft SMTP Server
+    elsif (/ with (?:Microsoft SMTP Server|mapi id) (?:\([^\)]+\) )?\d+\.\d+\.\d+\.\d+(?:$| )/) {
+      # Received: from EXC-DAG-02.global.net (10.45.252.152) by EXC-DAG-02.global.net
+      #  (10.45.252.152) with Microsoft SMTP Server (version=TLS1_2,
+      #  cipher=TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256) id 15.1.1261.35;
+      #  Mon, 29 Oct 2018 11:17:19 +0100
+      # Received: from AM5PR0402MB2836.eurprd04.prod.outlook.com
+      #  ([fe80::19bd:c588:dd17:5226]) by AM5PR0402MB2836.eurprd04.prod.outlook.com
+      #  ([fe80::19bd:c588:dd17:5226%6]) with mapi id 15.20.1943.023;
+      #  Wed, 5 Jun 2019 10:17:08 +0000
+      if (/^(\S+) \(\[?(${IP_ADDRESS})(?:%[A-Z0-9._~-]*)?\]?\) by (\S+) /) {
+        $helo = $1; $ip = $2; $by = $3; $id = ''; goto enough;
+      }
+    }
+
     elsif (/\[XMail /) { # bug 3791, bug 4053
       # Received: from list.brainbuzz.com (63.146.189.86:23198) by mx1.yourtech.net with [XMail 1.20 ESMTP Server] id <S72E> for <jason@ellingson.org.spamassassin.org> from <bounce-cscommunity-11965901@list.cramsession.com.spamassassin.org>; Sat, 18 Sep 2004 23:17:54 -0500
       # Received: from list.brainbuzz.com (63.146.189.86:23198) by mx1.yourtech.net (209.32.147.34:25) with [XMail 1.20 ESMTP Server] id <S72E> for <jason@ellingson.org.spamassassin.org> from <bounce-cscommunity-11965901@list.cramsession.com.spamassassin.org>; Sat, 18 Sep 2004 23:17:54 -0500
@@ -707,7 +727,7 @@ sub parse_received_line {
     # Wed, 24 Jul 2002 16:36:44 GMT
     if (/by (\S+\.hotmail\.msn\.com) /) {
       $by = $1;
-      /^(\S+) / and $ip = $1;
+      /^(${IP_ADDRESS}) / and $ip = $1;
       goto enough;
     }
 
@@ -828,10 +848,15 @@ sub parse_received_line {
     }
 
     # Received: from acecomms [202.83.84.95] by mailscan.acenet.net.au [202.83.84.27] with SMTP (MDaemon.PRO.v5.0.6.R) for <spamassassin-talk@lists.sourceforge.net>; Fri, 21 Feb 2003 09:32:27 +1000
-    if (/^(\S+) \[(${IP_ADDRESS})\] by (\S+) \[(\S+)\] with /) {
+    if (/^(\S+) \[(${IP_ADDRESS})\] by (\S+) \[${IP_ADDRESS}\] with /) {
       $mta_looked_up_dns = 1;
-      $helo = $1; $ip = $2;
-      $by = $4; # use the IP addr for "by", more useful?
+      $helo = $1; $ip = $2; $by = $3;
+      goto enough;
+    }
+
+    # Received: smtp510.aspkunden.de [(134.97.4.21)] by mail.aspemail.de (134.97.4.24) (MDaemon PRO v19.0.2)  with ESMTP id md50018233933.msg; Tue, 16 Jul 2019 11:39:22 +0200
+    if (/^(\S+) \[\((${IP_ADDRESS})\)\] by (\S+) \(${IP_ADDRESS}\) /) {
+      $helo = $1; $ip = $2; $by = $3;
       goto enough;
     }
 
@@ -873,12 +898,19 @@ sub parse_received_line {
       $helo = $1; $rdns = $2; $ip = $3; $by = $4; goto enough;
     }
 
+    # Identify fetch-from-server incidents:
+    # Fetchmail:
     # Received: from cabbage.jmason.org [127.0.0.1]
     # by localhost with IMAP (fetchmail-5.9.0)
     # for jm@localhost (single-drop); Thu, 13 Mar 2003 20:39:56 -0800 (PST)
-    if (/fetchmail/&&/^(\S+) (?:\[(${IP_ADDRESS})\] )?by (\S+) with \S+ \(fetchmail/) {
+    #
+    # Getmail:
+    # Received: from pop3.mail.dk (195.41.46.251) by loki.valhalla with POP3;
+    # 14 Apr 2010 11:14:29 -0000
+    #
+    if (/with (?:POP3|IMAP)/) {
       $self->found_pop_fetcher_sig();
-      return 0;		# skip fetchmail handovers
+      return 0;		# skip mail fetcher handovers
     }
 
     # Let's try to support a few qmailish formats in one;
@@ -916,14 +948,6 @@ sub parse_received_line {
       $ip = $1; $by = $2; goto enough;
     }
 
-    # Received: from po11.mit.edu [18.7.21.73]
-    # by stark.dyndns.tv with POP3 (fetchmail-5.9.7)
-    # for stark@localhost (single-drop); Tue, 18 Feb 2003 10:43:09 -0500 (EST)
-    # by po11.mit.edu (Cyrus v2.1.5) with LMTP; Tue, 18 Feb 2003 09:49:46 -0500
-    if (/ with POP3 /&&/^(\S+) \[(${IP_ADDRESS})\] by (\S+) with POP3 /) {
-      $rdns = $1; $ip = $2; $by = $3; goto enough;
-    }
-
     # Received: from snake.corp.yahoo.com(216.145.52.229) by x.x.org via smap (V1.3)
     # id xma093673; Wed, 26 Mar 03 20:43:24 -0600
     if (/ via smap /&&/^(\S+)\((${IP_ADDRESS})\) by (\S+) via smap /) {
@@ -974,10 +998,10 @@ sub parse_received_line {
     # Received: from customer254-217.iplannetworks.net (HELO AGAMENON) 
     # (baldusi@200.69.254.217 with plain) by smtp.mail.vip.sc5.yahoo.com with
     # SMTP; 11 Mar 2003 21:03:28 -0000
-    if (/^(\S+) \((?:HELO|EHLO) (\S*)\) \((\S+).*?\) by (\S+) with /) {
+    if (/^(\S+) \((?:HELO|EHLO) (\S*)\) \((?:(\S+)\@)?(${IP_ADDRESS}).*?\) by (\S+) with /) {
       $mta_looked_up_dns = 1;
-      $rdns = $1; $helo = $2; $ip = $3; $by = $4;
-      $ip =~ s/([^\@]*)\@//g and $ident = $1;	# remove IDENT lookups
+      $rdns = $1; $helo = $2; $ip = $4; $by = $5;
+      $ident = $3 if defined $3;
       goto enough;
     }
 
@@ -1016,7 +1040,7 @@ sub parse_received_line {
 
     # Received: from 01al10015010057.ad.bls.com ([90.152.5.141] [90.152.5.141])
     # by aismtp3g.bls.com with ESMTP; Mon, 10 Mar 2003 11:10:41 -0500
-    if (/^(\S+) \(\[(\S+)\] \[(\S+)\]\) by (\S+) with /) {
+    if (/^(\S+) \(\[(${IP_ADDRESS})\] \[(\S+)\]\) by (\S+) with /) {
       # not sure what $3 is ;)
       $helo = $1; $ip = $2; $by = $4;
       goto enough;
@@ -1117,7 +1141,7 @@ sub parse_received_line {
 
     # a synthetic header, generated internally:
     # Received: X-Originating-IP: 1.2.3.4
-    if (/^X-Originating-IP: (\S+)$/) {
+    if (/^X-Originating-IP: (${IP_ADDRESS})$/) {
       $ip = $1; $by = ''; goto enough;
     }
 
@@ -1186,8 +1210,11 @@ sub parse_received_line {
     # details of the handover described here, it's just qmail-scanner
     # logging a little more.
     if (/^\S+ by \S+ \(.{0,100}\) with qmail-scanner/) {
-      $envfrom =~ s/^\s*<*//gs; $envfrom =~ s/>*\s*$//gs;
-      $envfrom =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+      if (defined $envfrom) {
+        $envfrom =~ s/^\s*<*//gs;
+        $envfrom =~ s/>*\s*$//gs;
+        $envfrom =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+      }
       $self->{qmail_scanner_env_from} = $envfrom; # hack!
       return 0;
     }
@@ -1196,6 +1223,11 @@ sub parse_received_line {
     # id 18tOsg-0008FX-00; Thu, 13 Mar 2003 09:20:06 +0000
     if (/^\S+ by \S+ with local/) { return 0; }
 
+    # Local unix socket handover from Cyrus, tested with v2.3.14
+    # Received: from testintranator.net.vm ([unix socket])_ by testintranator.net.vm (Cyrus v2.3.14) with LMTPA;_ Tue, 21 Jul 2009 14:34:14 +0200
+    # Attention: Actually the received header is parsed as "testintranator.net.vm ([unix socket]) by testintranator.net.vm (Cyrus v2.3.14) with LMTPA", "from" is omitted.
+    if (/^\S+ \(\[unix socket\]\) by \S+ \(Cyrus v[0-9]*?\.[0-9]*?\.[0-9]*?\) with LMTPA/) { return 0; }
+
     # HANDOVERS WE KNOW WE CAN'T DEAL WITH: TCP transmission, but to MTAs that
     # just don't log enough info for us to use (ie. no IP address present).
     # Note: "return 0" is strongly recommended here, unless you're sure
@@ -1211,9 +1243,32 @@ sub parse_received_line {
     # from zimbramail.artsit.org.uk (unverified) by MAILSWEEP.birminghamartsit.org.uk (Clearswift SMTPRS 5.1.7) with ESMTP id <T78926b35f2c0a80003da8@MAILSWEEP.birminghamartsit.org.uk> for <discuss@lists.surbl.org>; Tue, 30 May 2006 15:56:15 +0100
     if (/^\S+ (?:(?:with|via|for) \S+|\(unverified\)) by\b/) { return 0; }
 
+    # from MjA3NDc4Mg (unknown) by ismtpd0001p1lon1.sendgrid.net (SG) with HTTP id aqHKNX2kSp-HiqspAa-uvw for <email@e.example.com>; Thu, 02 Apr 2020 07:53:55.516 +0000 (UTC)
+    if (/^\S+ \(unknown\) by \S+ \(SG\) with \b/) { return 0; }
+
+    # from localhost (example.com [local]) by example.com (OpenSMTPD) with ESMTPA id 5db34e0d for <email@example.com>; Tue, 7 Apr 2020 01:38:29 -0600 (MDT)
+    if (/^\S+ \(\S+ \[local\]\) by \S+ \(OpenSMTPD\) with \b/) { return 0; }
+
     # from DL1GSPMX02 (dl1gspmx02.gamestop.com) by email.ebgames.com (LSMTP for Windows NT v1.1b) with SMTP id <21.000575A0@email.ebgames.com>; Tue, 12 Sep 2006 21:06:43 -0500
     if (/\(LSMTP for/) { return 0; }
+
+    # from ([127.0.0.1]) with MailEnable ESMTP; Wed, 10 Jul 2019 10:29:59 +0300
+    if (/^\(\[${LOCALHOST}\]\) with MailEnable /) { return 0; }
+
+    # from facebook.com (RrlQsUbrndsQ6/zbJaSzSPcmy3GwqE5h6IukkE5GGBIJgonAFnoQE3L+9tv2TU3e 2401:db00:1110:50e8:face:0000:002f:0000)
+    #  by facebook.com with Thrift id 423753524b5011e9a83e248a0796a3b2-169bd530; Wed, 20 Mar 2019 13:39:29 -0700
+    if (/^facebook\.com \([^\)]+\) by facebook\.com with Thrift id \S+$/) { return 0; }
+
+    # from 384836569573 named unknown by gmailapi.google.com with HTTPREST; Wed, 6 Mar 2019 03:39:24 -0500
+    if (/^\S+ named \S+ by gmailapi\.google\.com with HTTPREST$/) { return 0; }
+
+    # from mail.payex.com id <B5b8f11e30004>; Wed, 05 Sep 2018 01:14:43 +0200
+    if (/^\S+ id \S+$/) { return 0; }
   
+    # from [<4124973-137966-3089@be2.maropost.com>] ([<4124973-137966-3089@be2.maropost.com>] helo=maropost.com)  by 643852-mailer2 (envelope-from 4124973-137966-3089@be2.maropost.com)
+    #  (Jetsend MTA 0.0.1 with ESMTP; Fri Sep 14 14:36:56 EDT 2018
+    if (/^\[<.*? \(Jetsend/) { return 0; }
+
     # if at this point we still haven't figured out the HELO string, see if we
     # can't just guess
     if (!$helo && /^(\S+)[^-A-Za-z0-9\.]/) { $helo = $1; }
@@ -1228,9 +1283,13 @@ sub parse_received_line {
     #  BY madman.mr.itd.umich.edu ID 434B508E.174A6.13932 ; 11 Oct 2005 01:41:34 -0400
     # Received: FROM [192.168.1.24] (s233-64-90-216.try.wideopenwest.com [64.233.216.90])
     #  BY hackers.mr.itd.umich.edu ID 434B5051.8CDE5.15436 ; 11 Oct 2005 01:40:33 -0400
-    if (/^(\S+) \((\S+) \[(${IP_ADDRESS})\]\) BY (\S+) ID (\S+)$/ ) {
+    # Received: FROM helo (1.2.3.4 [1.2.3.4]) BY xxx.com (Rockliffe SMTPRA 10.3.0)
+    #  WITH SMTP ID <B0065361981@xxx.com> FOR <foo@bar.net>; Tue, 6 Nov 2018 07:41:26 +0200
+
+    if (/^(\S+) \((\S+) \[(${IP_ADDRESS})\]\) BY (\S+) (?:\([^\)]+\) WITH SMTP )?ID <?(\S+?)>?(?: FOR <[^>]+>)?$/ ) {
       $mta_looked_up_dns = 1;
       $helo = $1; $rdns = $2; $ip = $3; $by = $4; $id = $5;
+      $rdns = '' if $rdns eq 'unverified';
       goto enough;
     }
   }
@@ -1295,7 +1354,10 @@ enough:
     }
   }
 
-  if ($rdns =~ /^unknown$/i) {
+  # Strip ending dot, Bug 7810
+  $rdns =~ s/\.+\z//;
+
+  if ($rdns =~ /^unknown$/i || $rdns =~ /^\[/) {
     $rdns = '';		# some MTAs seem to do this
   }
   
@@ -1310,19 +1372,23 @@ enough:
   # (only handles 'alternative form', not 'preferred form' - to be improved)
   $ip =~ s/^0*:0*:(?:0*:)*ffff:(\d+\.\d+\.\d+\.\d+)$/$1/i;
 
-  $envfrom =~ s/^\s*<*//gs; $envfrom =~ s/>*\s*$//gs;
   $by =~ s/\;$//;
 
   # ensure invalid chars are stripped.  Replace with '!' to flag their
   # presence, though.  NOTE: this means "[1.2.3.4]" IP addr HELO
   # strings, which are legit by RFC-2821, look like "!1.2.3.4!".
   # still useful though.
-  $ip =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
-  $rdns =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
-  $helo =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
-  $by =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
-  $ident =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
-  $envfrom =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+  my $strip_chars = qr/[\s\000\#\[\]\(\)\<\>\|]/;
+  $ip =~ s/$strip_chars/!/gs;
+  $rdns =~ s/$strip_chars/!/gs;
+  $helo =~ s/$strip_chars/!/gs;
+  $by =~ s/$strip_chars/!/gs;
+  $ident =~ s/$strip_chars/!/gs;
+  if (defined $envfrom) {
+    $envfrom =~ s/^\s*<*//gs;
+    $envfrom =~ s/>*\s*$//gs;
+    $envfrom =~ s/$strip_chars/!/gs;
+  }
 
   my $relay = {
     ip => $ip,
@@ -1369,7 +1435,10 @@ sub make_relay_as_string {
   # of entries must be preserved, so that regexps that assume that
   # e.g. "ip" comes before "helo" will still work.
   #
-  my $asstr = "[ ip=$relay->{ip} rdns=$relay->{rdns} helo=$relay->{helo} by=$relay->{by} ident=$relay->{ident} envfrom=$relay->{envfrom} intl=0 id=$relay->{id} auth=$relay->{auth} msa=0 ]";
+
+  # we could mark envfrom as "undef" if missing? dunno if needed?
+  my $envfrom = $relay->{envfrom} || '';
+  my $asstr = "[ ip=$relay->{ip} rdns=$relay->{rdns} helo=$relay->{helo} by=$relay->{by} ident=$relay->{ident} envfrom=$envfrom intl=0 id=$relay->{id} auth=$relay->{auth} msa=0 ]";
   dbg("received-header: parsed as $asstr");
   $relay->{as_string} = $asstr;
 }
@@ -1378,13 +1447,13 @@ sub make_relay_as_string {
 # spamcop does this, and it's a great idea ;)
 sub found_pop_fetcher_sig {
   my ($self) = @_;
-  if ($self->{allow_fetchmail_markers}) {
-    dbg("received-header: found fetchmail marker, restarting parse");
+  if ($self->{allow_mailfetch_markers}) {
+    dbg("received-header: found mail fetcher marker, restarting parse");
     $self->{relays_trusted} = [ ];
     $self->{relays_internal} = [ ];
     $self->{relays_external} = [ ];
   } else {
-    dbg("received-header: found fetchmail marker outside trusted area, ignored");
+    dbg("received-header: found mail fetcher marker outside trusted area, ignored");
   }
 }
 
