@@ -38,11 +38,10 @@ use Mail::SpamAssassin::Logger;
 use Mail::SpamAssassin::Timeout;
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
-use vars qw(@ISA);
-@ISA = qw(Mail::SpamAssassin::Plugin);
+our @ISA = qw(Mail::SpamAssassin::Plugin);
 
 # constructor: register the eval rule
 sub new {
@@ -114,6 +113,14 @@ Same as C<whitelist_from_spf>, but used for the default whitelist entries
 in the SpamAssassin distribution.  The whitelist score is lower, because
 these are often targets for spammer spoofing.
 
+=item unwhitelist_from_spf user@example.com
+
+Used to remove a C<whitelist_from_spf> or C<def_whitelist_from_spf> entry. 
+The specified email address has to match exactly the address previously used.
+
+Useful for removing undesired default entries from a distributed configuration
+by a local or site-specific configuration or by C<user_prefs>.
+
 =cut
 
   push (@cmds, {
@@ -126,6 +133,24 @@ these are often targets for spammer spoofing.
     type => $Mail::SpamAssassin::Conf::CONF_TYPE_ADDRLIST
   });
 
+  push (@cmds, {
+    setting => 'unwhitelist_from_spf',
+    type => $Mail::SpamAssassin::Conf::CONF_TYPE_ADDRLIST,
+    code => sub {
+      my ($self, $key, $value, $line) = @_;
+      unless (defined $value && $value !~ /^$/) {
+        return $Mail::SpamAssassin::Conf::MISSING_REQUIRED_VALUE;
+      }
+      unless ($value =~ /^(?:\S+(?:\s+\S+)*)$/) {
+        return $Mail::SpamAssassin::Conf::INVALID_VALUE;
+      }
+      $self->{parser}->remove_from_addrlist('whitelist_from_spf',
+                                        split (/\s+/, $value));
+      $self->{parser}->remove_from_addrlist('def_whitelist_from_spf',
+                                        split (/\s+/, $value));
+    }
+  });
+
 =back
 
 =head1 ADMINISTRATOR OPTIONS
@@ -391,7 +416,7 @@ sub _check_spf {
 
 	# http://www.openspf.org/RFC_4408#header-field
 	# wtf - for some reason something is sticking an extra space between the header name and field value
-	if ($hdr =~ /^received-spf:\s*(pass|neutral|(?:soft)?fail|none)\b(?:.*\bidentity=(\S+?);?\b)?/i) {
+	if ($hdr =~ /^received-spf:\s*(pass|neutral|(?:soft)?fail|(?:temp|perm)error|none)\b(?:.*\bidentity=(\S+?);?\b)?/i) {
 	  my $result = lc($1);
 
 	  my $identity = '';	# we assume it's a mfrom check if we can't tell otherwise
@@ -418,6 +443,8 @@ sub _check_spf {
 	  $scanner->{"spf_${identity}none"} = 0;
 	  $scanner->{"spf_${identity}fail"} = 0;
 	  $scanner->{"spf_${identity}softfail"} = 0;
+	  $scanner->{"spf_${identity}temperror"} = 0;
+	  $scanner->{"spf_${identity}permerror"} = 0;
 	  $scanner->{"spf_${identity}failure_comment"} = undef;
 
 	  # and the result
@@ -439,7 +466,7 @@ sub _check_spf {
         # Authentication-Results: mail.example.com; SPF=none smtp.mailfrom=example.org (comment)
 
         my $tmphdr = $1;
-        if ($tmphdr =~ /^(pass|neutral|(?:hard|soft)?fail|none)(?:[^;]*?\bsmtp\.(\S+)\s*=[^;]+)?/i) {
+        if ($tmphdr =~ /^(pass|neutral|(?:hard|soft)?fail|(?:temp|perm)error|none)(?:[^;]*?\bsmtp\.(\S+)\s*=[^;]+)?/i) {
           my $result = lc($1);
           $result = 'fail'  if $result eq 'hardfail';  # RFC5451 permits this
 
@@ -467,6 +494,8 @@ sub _check_spf {
           $scanner->{"spf_${identity}none"} = 0;
           $scanner->{"spf_${identity}fail"} = 0;
           $scanner->{"spf_${identity}softfail"} = 0;
+          $scanner->{"spf_${identity}temperror"} = 0;
+          $scanner->{"spf_${identity}permerror"} = 0;
           $scanner->{"spf_${identity}failure_comment"} = undef;
 
           # and the result
@@ -727,7 +756,7 @@ sub _check_spf {
     elsif ($result eq 'temperror') { $scanner->{spf_helo_temperror} = 1; }
     elsif ($result eq 'error') { $scanner->{spf_helo_temperror} = 1; }
 
-    if ($result eq 'fail') {	# RFC 4408 6.2
+    if ($result eq 'fail') {	# RFC 7208 6.2
       $scanner->{spf_helo_failure_comment} = "SPF failed: $comment";
     }
   } else {
@@ -740,7 +769,7 @@ sub _check_spf {
     elsif ($result eq 'temperror') { $scanner->{spf_temperror} = 1; }
     elsif ($result eq 'error') { $scanner->{spf_temperror} = 1; }
 
-    if ($result eq 'fail') {	# RCF 4408 6.2
+    if ($result eq 'fail') {	# RFC 7208 6.2
       $scanner->{spf_failure_comment} = "SPF failed: $comment";
     }
   }
@@ -809,7 +838,7 @@ sub _check_spf_whitelist {
   $self->_get_sender($scanner) unless $scanner->{sender_got};
 
   unless ($scanner->{sender}) {
-    dbg("spf: spf_whitelist_from: could not find useable envelope sender");
+    dbg("spf: spf_whitelist_from: could not find usable envelope sender");
     return;
   }
 
@@ -847,7 +876,7 @@ sub _check_def_spf_whitelist {
   $self->_get_sender($scanner) unless $scanner->{sender_got};
 
   unless ($scanner->{sender}) {
-    dbg("spf: def_spf_whitelist_from: could not find useable envelope sender");
+    dbg("spf: def_spf_whitelist_from: could not find usable envelope sender");
     return;
   }
 
